/*---------------------------------------------------------------------------
  This code is part of the book
  LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
  Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
  Hellenic Academic On-line Publications, 2015
----------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
  A SIMPLE RULE BASED SYSTEM  
----------------------------------------------------------------------------*/

% Operator Definitions for a more natutal rule Syntax
:-op(800,fx,if).
:-op(700,xfx,then).
:-op(400,yfx,and).
:-op(300,yfx,or).
:-op(200,fy,not).

/*---------------------------------------------------------------------------
%%% backward Chaining
%%% prove_bc/1
%%% prove_bc(Goal).
%%% Succeeds if Goal can be proved wrt the rules encoded in the system
----------------------------------------------------------------------------*/

% CHECK IF THERE IS SOMETING WE KNOW.
prove_bc(Goal):-
	fact(Goal),
	write(' Prooved:: '), write(Goal),nl.

% BACKWARD CHAINING :: TO PROVE GOAL PROVE THE CONDITIONS.
prove_bc(Goal):-
	if Condition then Goal, 
	write('The goal: '), write(Goal), write(' matches the rule::'),nl,
	write(' IF '),write(Condition),write( ' then '), write(Goal), nl,
	write('**Rule Start**'),nl,
	write('Trying to prove the conditions of the rule:: '),nl,write(Condition),nl, 
	prove_bc(Condition),
	write('Success of the rule::'),
	write(' IF '),write(Condition),write( ' then '), write(Goal), nl,
	write('**Rule End**'),nl.

% CONJUNCTION
prove_bc(G1 and G2):-
	write('Trying to first to prove:: '),write(G1),nl,write(' AND then '), write(G2),nl, 
	prove_bc(G1),
	prove_bc(G2).

% DISJUNCTION
prove_bc(G1 or G2):-
	write('Trying to prove:: '),nl,write(G1),write(' OR '), write(G2),nl, 
	prove_bc(G1)
          ;
	prove_bc(G2).

% DANGEROUS NEGATION
% IF I CANNOT PROVE IT THEN IT IS NOT TRUE
prove_bc(not Goal):-
	write('Trying to prove:: NOT '),nl,write(Goal),nl, 
	not(prove_bc(Goal)).

/*---------------------------------------------------------------------------
  fact/1
  fact(FACT)
  Something is a fact if it is in the knowledge base, i.e in frames
  There is a special case for test/1 which checks whether something  
  holds (arithemtic tests, user-defined predicates, etc). 
----------------------------------------------------------------------------*/


fact(prolog(TEST)):-
	call(TEST).

fact(F):-
	clause(F,_),!, %% making sure that there is a corresponding clause. 
	F.	

